(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.Toast = {}));
})(this, (function (exports) { 'use strict';

    /**
     * Copyright (c) 2022 Peyton Gasink
     * Distributed under MIT License.
     *
     * This file contains all the necessary scripting to programmatically
     * generate Bootstrap toasts. It first inserts a container at the bottom
     * of the DOM, then fills a toast template and inserts it into the container.
     *
     * Configuration options are also provided for toast placement, light & dark themes,
     * and the maximum number of toasts allowed on the page at a given time.
     */
    /** Container that generated toasts will be inserted into. */
    var TOAST_CONTAINER = document.createElement("div");
    TOAST_CONTAINER.id = "toastContainer";
    TOAST_CONTAINER.className = "toast-container position-fixed top-0 end-0";
    TOAST_CONTAINER.setAttribute("aria-live", "polite");
    document.body.appendChild(TOAST_CONTAINER);
    /** HTML markup for the toast template. */
    var TOAST_TEMPLATE = document.createElement("div");
    TOAST_TEMPLATE.className = "toast";
    TOAST_TEMPLATE.setAttribute("role", "status");
    TOAST_TEMPLATE.setAttribute("aria-live", "polite");
    TOAST_TEMPLATE.setAttribute("aria-atomic", "true");
    TOAST_TEMPLATE.setAttribute("data-bs-autohide", "false");
    TOAST_TEMPLATE.innerHTML = "\n        <div class=\"toast-header\">\n            <span class=\"status-icon bi me-2\" aria-hidden=\"true\"></span>\n            <strong class=\"me-auto toast-title\"></strong>\n            <small class=\"timer\" aria-hidden=\"true\"></small>\n            <button type=\"button\" class=\"btn-close ms-2\" data-bs-dismiss=\"toast\" aria-label=\"Close\"></button>\n        </div>\n        <div class=\"toast-body\"></div>";
    /** Defines the valid status options for toasts. */
    exports.TOAST_STATUS = void 0;
    (function (TOAST_STATUS) {
        TOAST_STATUS[TOAST_STATUS["SUCCESS"] = 1] = "SUCCESS";
        TOAST_STATUS[TOAST_STATUS["DANGER"] = 2] = "DANGER";
        TOAST_STATUS[TOAST_STATUS["WARNING"] = 3] = "WARNING";
        TOAST_STATUS[TOAST_STATUS["INFO"] = 4] = "INFO";
    })(exports.TOAST_STATUS || (exports.TOAST_STATUS = {}));
    window.TOAST_STATUS = exports.TOAST_STATUS;
    /** Defines the valid placement options for the toast container. */
    exports.TOAST_PLACEMENT = void 0;
    (function (TOAST_PLACEMENT) {
        TOAST_PLACEMENT[TOAST_PLACEMENT["TOP_LEFT"] = 1] = "TOP_LEFT";
        TOAST_PLACEMENT[TOAST_PLACEMENT["TOP_CENTER"] = 2] = "TOP_CENTER";
        TOAST_PLACEMENT[TOAST_PLACEMENT["TOP_RIGHT"] = 3] = "TOP_RIGHT";
        TOAST_PLACEMENT[TOAST_PLACEMENT["MIDDLE_LEFT"] = 4] = "MIDDLE_LEFT";
        TOAST_PLACEMENT[TOAST_PLACEMENT["MIDDLE_CENTER"] = 5] = "MIDDLE_CENTER";
        TOAST_PLACEMENT[TOAST_PLACEMENT["MIDDLE_RIGHT"] = 6] = "MIDDLE_RIGHT";
        TOAST_PLACEMENT[TOAST_PLACEMENT["BOTTOM_LEFT"] = 7] = "BOTTOM_LEFT";
        TOAST_PLACEMENT[TOAST_PLACEMENT["BOTTOM_CENTER"] = 8] = "BOTTOM_CENTER";
        TOAST_PLACEMENT[TOAST_PLACEMENT["BOTTOM_RIGHT"] = 9] = "BOTTOM_RIGHT";
    })(exports.TOAST_PLACEMENT || (exports.TOAST_PLACEMENT = {}));
    window.TOAST_PLACEMENT = exports.TOAST_PLACEMENT;
    /** Defines the valid options for toast themes. */
    exports.TOAST_THEME = void 0;
    (function (TOAST_THEME) {
        TOAST_THEME[TOAST_THEME["LIGHT"] = 1] = "LIGHT";
        TOAST_THEME[TOAST_THEME["DARK"] = 2] = "DARK";
    })(exports.TOAST_THEME || (exports.TOAST_THEME = {}));
    window.TOAST_THEME = exports.TOAST_THEME;
    /** Defines the valid options for toast header timers. */
    exports.TOAST_TIMERS = void 0;
    (function (TOAST_TIMERS) {
        TOAST_TIMERS[TOAST_TIMERS["DISABLED"] = 0] = "DISABLED";
        TOAST_TIMERS[TOAST_TIMERS["ELAPSED"] = 1] = "ELAPSED";
        TOAST_TIMERS[TOAST_TIMERS["COUNTDOWN"] = 2] = "COUNTDOWN";
    })(exports.TOAST_TIMERS || (exports.TOAST_TIMERS = {}));
    window.TOAST_TIMERS = exports.TOAST_TIMERS;
    var Toast = /** @class */ (function () {
        function Toast() {
        }
        /**
         * Shorthand function for quickly setting multiple global toast configurations.
         * @param {ToastConfigOptions} options Object containing all the desired toast options.
         */
        Toast.configure = function (options) {
            this.setMaxCount(options === null || options === void 0 ? void 0 : options.maxToasts);
            this.setPlacement(options === null || options === void 0 ? void 0 : options.placement);
            this.setTheme(options === null || options === void 0 ? void 0 : options.theme);
            this.enableTimers(options === null || options === void 0 ? void 0 : options.enableTimers);
            this.enableQueue(options === null || options === void 0 ? void 0 : options.enableQueue);
        };
        /**
         * Sets the maximum number of toasts allowed on the page at once.
         * @param {number} maxToasts Maximum number of toasts allowed on the page at once.
         */
        Toast.setMaxCount = function (maxToasts) {
            if (maxToasts !== null) {
                if (maxToasts > 0) {
                    this.maxToastCount = maxToasts;
                }
                else {
                    console.error("The maximum number of toasts must be greater than 0. Reverting to default.");
                }
            }
        };
        /**
         * Sets the toast container's placement.
         * @param {TOAST_PLACEMENT} placement Placement of the toast container.
         */
        Toast.setPlacement = function (placement) {
            TOAST_CONTAINER.className = "toast-container position-fixed";
            switch (placement) {
                case exports.TOAST_PLACEMENT.TOP_LEFT:
                    TOAST_CONTAINER.classList.add("top-0", "start-0");
                    break;
                case exports.TOAST_PLACEMENT.TOP_CENTER:
                    TOAST_CONTAINER.classList.add("top-0", "start-50", "translate-middle-x");
                    break;
                case exports.TOAST_PLACEMENT.TOP_RIGHT:
                    TOAST_CONTAINER.classList.add("top-0", "end-0");
                    break;
                case exports.TOAST_PLACEMENT.MIDDLE_LEFT:
                    TOAST_CONTAINER.classList.add("top-50", "start-0", "translate-middle-y");
                    break;
                case exports.TOAST_PLACEMENT.MIDDLE_CENTER:
                    TOAST_CONTAINER.classList.add("top-50", "start-50", "translate-middle");
                    break;
                case exports.TOAST_PLACEMENT.MIDDLE_RIGHT:
                    TOAST_CONTAINER.classList.add("top-50", "end-0", "translate-middle-y");
                    break;
                case exports.TOAST_PLACEMENT.BOTTOM_LEFT:
                    TOAST_CONTAINER.classList.add("bottom-0", "start-0");
                    break;
                case exports.TOAST_PLACEMENT.BOTTOM_CENTER:
                    TOAST_CONTAINER.classList.add("bottom-0", "start-50", "translate-middle-x");
                    break;
                case exports.TOAST_PLACEMENT.BOTTOM_RIGHT:
                    TOAST_CONTAINER.classList.add("bottom-0", "end-0");
                    break;
                default:
                    TOAST_CONTAINER.classList.add("top-0", "end-0");
                    break;
            }
        };
        /**
         * Sets the toasts' theme to light or dark. If unset, they will follow OS light/dark preference.
         * @param {TOAST_THEME} theme The toast theme.
         */
        Toast.setTheme = function (theme) {
            if (theme === void 0) { theme = null; }
            var header = TOAST_TEMPLATE.querySelector(".toast-header");
            var close = header.querySelector(".btn-close");
            switch (theme) {
                case exports.TOAST_THEME.LIGHT:
                    TOAST_TEMPLATE.style.backgroundColor = "var(--body-bg-color-light)";
                    TOAST_TEMPLATE.style.color = "var(--text-color-light)";
                    header.style.backgroundColor = "var(--header-bg-color-light)";
                    header.style.color = "var(--header-color-light)";
                    close.style.filter = "unset";
                    break;
                case exports.TOAST_THEME.DARK:
                    TOAST_TEMPLATE.style.backgroundColor = "var(--body-bg-color-dark)";
                    TOAST_TEMPLATE.style.color = "var(--text-color-dark)";
                    header.style.backgroundColor = "var(--header-bg-color-dark)";
                    header.style.color = "var(--header-color-dark)";
                    close.style.filter = "invert(1) grayscale(100%) brightness(200%)";
                    break;
                default:
                    TOAST_TEMPLATE.removeAttribute("style");
                    header.removeAttribute("style");
                    close.removeAttribute("style");
                    break;
            }
        };
        /**
         * Sets whether timers in the toast header will display elapsed time or a countdown.
         * Timers display elapsed time by default.
         * @param type The timer type.
         */
        Toast.enableTimers = function (type) {
            this.timersEnabled = type;
        };
        /**
         * Enables or disables toasts queueing after the maximum toast count is reached.
         * Queuing is enabled by default.
         * @param {boolean} enabled Controls whether queue is enabled.
         */
        Toast.enableQueue = function (enabled) {
            if (enabled === void 0) { enabled = true; }
            this.queueEnabled = enabled;
            // Empty the queue once it's disabled.
            if (!enabled)
                this.queue = [];
        };
        /**
         * Endpoint to generate Bootstrap toasts from a template and insert their HTML onto the page,
         * run timers for each's elapsed time since appearing, and remove them from the
         * DOM after they are hidden. Caps toast count at maxToastCount.
         * @param {ToastOptions} toastOptions Object containing all the desired toast options.
         */
        Toast.create = function (toastOptions) {
            var toastEl = TOAST_TEMPLATE.cloneNode(true);
            var toastTitle = toastEl.querySelector(".toast-title");
            toastTitle.innerText = toastOptions.title;
            var toastBody = toastEl.querySelector(".toast-body");
            toastBody.innerHTML = toastOptions.message;
            this.setStatus(toastEl, toastOptions.status);
            // Add toast to the queue if it would exceed maxToastCount
            if (this.currentToastCount >= this.maxToastCount) {
                if (!this.queueEnabled)
                    return;
                var toastToQueue = {
                    toast: toastEl,
                    timeout: toastOptions.timeout
                };
                this.queue.push(toastToQueue);
                return;
            }
            var toastInfo = {
                toast: toastEl,
                timeout: toastOptions.timeout
            };
            this.render(toastInfo);
        };
        /**
         * Sets the status icon and modifies ARIA properties if the context necessitates it
         * @param {HTMLElement} toastEl The HTML of the toast being modified.
         * @param {TOAST_STATUS} status The integer value representing the toast's status.
         */
        Toast.setStatus = function (toastEl, status) {
            var statusIcon = toastEl.querySelector(".status-icon");
            switch (status) {
                case exports.TOAST_STATUS.SUCCESS:
                    statusIcon.classList.add("text-success", "bi-check-circle-fill");
                    break;
                case exports.TOAST_STATUS.DANGER:
                    statusIcon.classList.add("text-danger", "bi-x-circle-fill");
                    toastEl.setAttribute("role", "alert");
                    toastEl.setAttribute("aria-live", "assertive");
                    break;
                case exports.TOAST_STATUS.WARNING:
                    statusIcon.classList.add("text-warning", "bi-exclamation-circle-fill");
                    toastEl.setAttribute("role", "alert");
                    toastEl.setAttribute("aria-live", "assertive");
                    break;
                case exports.TOAST_STATUS.INFO:
                    statusIcon.classList.add("text-info", "bi-info-circle-fill");
                    break;
                default:
                    statusIcon.classList.add("d-none");
                    break;
            }
        };
        /**
         * Inserts toast HTML onto page and sets up for toast deletion.
         * @param {ToastData} toastInfo The toast object to be rendered.
         */
        Toast.render = function (toastInfo) {
            var _this = this;
            // If the timeout is 0, the toast must be dismissed manually
            if (toastInfo.timeout > 0) {
                toastInfo.toast.setAttribute("data-bs-delay", toastInfo.timeout.toString());
                toastInfo.toast.setAttribute("data-bs-autohide", "true");
            }
            this.renderTimer(toastInfo);
            TOAST_CONTAINER.appendChild(toastInfo.toast);
            // Initialize Bootstrap 5's toast plugin
            var bsToast = new bootstrap.Toast(toastInfo.toast);
            bsToast.show();
            this.currentToastCount++;
            // When the toast hides, remove it from the DOM
            toastInfo.toast.addEventListener("hidden.bs.toast", function () {
                TOAST_CONTAINER.removeChild(toastInfo.toast);
                _this.currentToastCount--;
                if (_this.queueEnabled &&
                    _this.queue.length > 0 &&
                    _this.currentToastCount < _this.maxToastCount) {
                    var queuedToast = _this.queue.shift();
                    _this.render(queuedToast);
                }
            });
        };
        /**
         * Handles the rendering of the timer in the toast header.
         * @param toastInfo The toast object to be rendered.
         */
        Toast.renderTimer = function (toastInfo) {
            var timer = toastInfo.toast.querySelector(".timer");
            switch (this.timersEnabled) {
                case exports.TOAST_TIMERS.ELAPSED: {
                    timer.innerText = "just now";
                    // Start a timer that updates the text of the time indicator every minute
                    // Initially set to 1 because for the first minute the indicator reads "just now"
                    var minutes_1 = 1;
                    var elapsedTimer_1 = setInterval(function () {
                        timer.innerText = "".concat(minutes_1, "m ago");
                        minutes_1++;
                    }, 60 * 1000);
                    // When the toast hides, delete its timer instance
                    toastInfo.toast.addEventListener("hidden.bs.toast", function () {
                        clearInterval(elapsedTimer_1);
                    });
                    break;
                }
                case exports.TOAST_TIMERS.COUNTDOWN: {
                    if (toastInfo.timeout > 0) {
                        // Start a timer that updates the text of the time indicator every minute
                        // Initially set to 1 because for the first minute the indicator reads "just now"
                        var seconds_1 = toastInfo.timeout / 1000;
                        timer.innerText = "".concat(seconds_1, "s");
                        var countdownTimer_1 = setInterval(function () {
                            timer.innerText = "".concat(seconds_1 - 1, "s");
                            seconds_1--;
                        }, 1000);
                        // When the toast hides, delete its timer instance
                        toastInfo.toast.addEventListener("hidden.bs.toast", function () {
                            clearInterval(countdownTimer_1);
                        });
                    }
                    break;
                }
                default: {
                    var toastHeader = toastInfo.toast.querySelector(".toast-header");
                    toastHeader.removeChild(timer);
                    break;
                }
            }
        };
        /** Maximum amount of toasts to be allowed on the page at once. */
        Toast.maxToastCount = 4;
        /** Number of toasts currently rendered on the page. */
        Toast.currentToastCount = 0;
        /** Controls whether toasts will have elapsed or countdown timers. */
        Toast.timersEnabled = exports.TOAST_TIMERS.ELAPSED;
        /** Controls whether to queue toasts that exceed the maximum toast count. */
        Toast.queueEnabled = true;
        Toast.queue = [];
        return Toast;
    }());
    window.Toast = Toast;

    exports.Toast = Toast;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=bootstrap-toaster.js.map
